"""5.1.4 Update endi_payment

Revision ID: aaaaaaaaaaaa
Revises: None
Create Date: 2020-03-18 16:40:19.616393

"""

# revision identifiers, used by Alembic.
revision = 'aaaaaaaaaaaa'
down_revision = None

from alembic import op
import sqlalchemy as sa


def update_database_structure():
    # ### commands auto generated by Alembic - please adjust! ###
    # ### end Alembic commands ###
    pass

def migrate_datas():
    pass

def upgrade():
    update_database_structure()
    migrate_datas()


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
